<?php // $Id$

class time_entry_qb extends time_entry {
  var $customer;
  var $source = 'drupal';
  var $qb_txn_id;
  var $qb_edit_sequence;

  function __construct($row = NULL) {
    // Before calling our parent's constructor, augment time info with qb data.
    if ($row) {
      $row = (array) $row;
      if ($row['teid']) {
        $row = array_merge($row, db_fetch_array(db_query("SELECT * 
          FROM {time_entry_qb} WHERE teid = %d", $row['teid'])));
      }
    }
    parent::__construct($row);
  }

  function save($values = NULL) {
    $update = isset($this->teid) ? 'teid' : NULL;

    // Save the time entry data in the usual place.
    parent::save($values);

    // Save the QB-only fields into our own table.
    drupal_write_record('time_entry_qb', $this, $update);
  }

  function validate() {

    // Require the current user to have a quickbooks relationship.
    if (!$this->qb_user()) return FALSE;

    // Require some mechanism for retreiving a customer mapping.
    if (!$this->options['qb_customer']['select']) {
       if (!$this->qb_customer()) return FALSE;
    }

    return TRUE;
  }

  function activities() {
    $activities = array();
    if ($qdid = $this->options['qb_itemservice']['qdid']) {
      $activities = qb_data_values_array($qdid);
      foreach ($this->options['qb_itemservice']['activities'] as $key => $on) {
        if (!$on) unset($activities[$key]);
      }
    }
    return $activities;
  }

  function form() {
    $form = parent::form();
    if ($this->customer || (!$this->qb_customer() && $this->options['qb_customer']['select'])) {
      $form['customer'] = array(
        '#type' => 'select',
        '#title' => t('Customer'),
        '#options' => qb_data_values_array(1), // TODO hard-coded.
        '#default_value' => $this->customer,
        '#weight' => -1,
      );
    }
    return $form;
  }

  function options_form() {
    $form = array();

    // Map activities to ItemService list items.
    $form['qb_itemservice'] = array(
      '#type' => 'fieldset',
      '#title' => t('Service items'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['qb_itemservice']['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Map activities to Quickbooks Service Items'),
      '#default_value' => $this->options['qb_itemservice']['enabled'],
      '#description' => t('Service items are stored in Quickbooks, and reference activities such as labor, consulting, or professional fees.  If you want to create billable time entries, you need to enable this.'),
    );
    //if ($qdid = $this->options['qb_itemservice']['qdid']) {
    if (TRUE) {
      $qdid = 2; // TODO definitely not a guarantee.
      $form['qb_itemservice']['qdid'] = array('#type' => 'value', '#value' => $qdid);
      $form['qb_itemservice']['activities'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Enabled service items'),
        '#description' => t('Check the boxes for all of the service items you would like to include on time tracking forms.'),
      );
      foreach (qb_data_values_array($qdid) as $qdvid => $name) {
        $form['qb_itemservice']['activities'][$qdvid] = array(
          '#type' => 'checkbox',
          '#title' => $name,
          '#default_value' => isset($this->options['qb_itemservice']['activities'][$qdvid]) ? $this->options['qb_itemservice']['activities'][$qdvid] : 1,
        );
      }
    }

    // Set Customer mappings. Must be from the CustomerList.
    $form['qb_customer'] = array(
      '#type' => 'fieldset',
      '#title' => t('Customers'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#description' => t('Time entries sent to Quickbooks for billing purposes must be associated with a customer.  If none of the following methods result in a valid customer mapping, the time entry form will not appear.'),
    );
    $form['qb_customer']['select'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow user to select from a list of customers.'),
      '#default_value' => $this->options['select'],
      '#description' => t('Include a select list of active customers on the time tracking form.  This means that any user who can track time will be able to see a the entire customer list, which might be a bad thing.'),
    );

    if (module_exists('qb_field')) {
			if ($qb_fields = qb_field_list('Customer')) {
        $fields = array();
        foreach ($qb_fields as $field) {
          $fields[$field['field_name']] = $field['widget']['label'];
        }
        $form['qb_customer']['fields'] = array(
          '#type' => 'checkboxes',
					'#title' => t('Content fields'),
          '#default_value' => isset($this->options['qb_customer']['fields']) ? $this->options['qb_customer']['fields'] : array(),
          '#options' => $fields,
          '#description' => t('Nodes that include these fields will be eligible for time tracking, the customer will be derived from these fields.'),
        );
      }
      else {
        $form['qb_customer']['qb_customer_fields'] = array(
          '#type' => 'markup',
          '#value' => t('If you add a Quickbooks record field to one or more content types, and use "Customer" as the Quickbooks record type, you will be able to track time against nodes that contain that field.'),
        );
      }
    }
    else {
      $form['qb_customer']['qb_customer_select'] = array(
        '#type' => 'checkbox',
        '#title' => t('Allow user to select from a list of customers.'),
        '#default_value' => $this->options['qb_customer_select'],
        '#description' => t('Include a select list of active customers on the '),
      );
    }

    // Set EntityRef mappings. Can include Employees, Vendors, or OtherNames.
    $form['qb_entity'] = array(
      '#type' => 'fieldset',
      '#title' => t('Employees and Vendors'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#description' => t('Time entries sent to Quickbooks for billing purposes must include a name from the Employees, Vendors, or Other Names list.  Only Drupal users who are linked to a Quickbooks contact entry can use this time tracking profile.  Please make sure you have the !link module configured to handle this properly.', array('!link' => l('Quickbooks user', 'admin/settings/qb/user'))),
    );

    // Set Class entry.
    $form['qb_class'] = array(
      '#type' => 'fieldset',
      '#title' => t('Class tracking'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['qb_class']['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use class tracking'),
      '#default_value' => $this->options['qb_class']['enabled'],
      '#description' => t('Classes can be used to separate transactions into meaningful categories. In QuickBooks, class tracking is off by default.  Check this options if you are using classes and would like to assign time tracking entries with a class.')
    );
    if ($qdid = $this->options['qb_class']['qdid']) {
      $form['qb_class']['qdvid'] = array(
        '#type' => 'select',
        '#options' => array('' => '') + qb_data_values($qdid),
        '#default_value' => $this->options['qb_class']['qdvid'],
      );
    }

    return $form;
  }

  function options_form_submit($values) {
    // Save the new entry.

    parent::options_form_submit($values);
  }

  function qb_activity() {
    if (isset($this->_qb_activity)) return $this->_qb_activity;

    $info = qb_data_value($this->activity);
    return $this->_qb_activity = $info->qb_list_id;
  }

  function qb_user() {
    if (isset($this->_qb_user)) return $this->_qb_user;

    $user = $this->user();
    foreach (array('employee', 'vendor', 'othernames') as $type) {
      if (isset($user->qb_user[$type])) {
        $info = qb_data_value($user->qb_user[$type]);
        return $this->_qb_user = $info->qb_list_id;
      }
    }
  }

  function set_source($source = NULL) {
    if (!$source) $source = 'drupal';
    $this->source = $source;
  }

  function set_customer($qdvid = NULL) {
    if ($qdvid) {
      $this->customer = $qdvid;
    }
    else {
      // Right now, the only way to derive customers is from field values.
      if (!$node = $this->node()) return;

      // Relying on the qb_data module.  Look for CCK fields.
      if (!$fields = $this->options['qb_customer']['fields']) return;
      foreach ($fields as $field => $status) {
        if ($status) {
          if ($qdvid = $node->{$field}[0]['qdvid']) {
            $this->customer = $qdvid;
            return;
          }
        }
      }
    }
  }
 
  function qb_customer() {
    $this->set_customer();
    if (isset($this->_qb_customer)) return $this->_qb_customer;
    $info = qb_data_value($this->customer);
    return $this->_qb_customer = $info->qb_list_id;
  }

  function qb_date() {
    return date('Y-m-d', $this->timestamp);
  }

  function qb_duration() {
    $hours = floor($this->duration());
    $minutes = floor(($this->duration() - $hours) * 60);
    return 'PT'. $hours .'H' . $minutes . 'M';
  }

  function qb_billable() {
    $enum = array( -1 => 'HasBeenBilled', 0 => 'NotBillable', 1 => 'Billable');
    return $enum[$this->billable];
  }

  function qb_note() {
    return strip_tags($this->note);
  }
}
